#ifndef __EMITTER_H__
#define __EMITTER_H__
#include <vector>
#include <QTime>
#include "ngl/Vector.h"
#include "ngl/VBOPrimitives.h"

#include "Particle.h"


class Emitter
{
public :

	/// @brief ctor
	/// @param _pos the position of the emitter
	/// @param _numParticles the number of particles to create
	Emitter(
					 ngl::Vector _pos,
					 std::string _texture,
					 int _numParticles
				 );
	/// @brief a method to update each of the particles contained in the system
	void update();
	/// @brief a method to draw all the particles contained in the system
	void draw();
	inline GLuint getTextureID() const {return m_textureID;}
private :
	/// @brief the position of the emitter
	ngl::Vector m_pos;
	/// @brief the number of particles
	int m_numParticles;
	/// @brief the container for the particles
	std::vector <Particle> m_particles;
	/// @brief the container for the points passed to GL
	std::vector <GLfloat> m_particlePositions;
	/// @brief the texture ID for the emitters point sprite
	GLuint m_textureID;


};


#endif

