#include "Emitter.h"
#include "ngl/Texture.h"

/// @brief ctor
/// @param _pos the position of the emitter
/// @param _numParticles the number of particles to create
Emitter::Emitter(
									ngl::Vector _pos,
									std::string _texture,
									int _numParticles
								 )
{
	for (int i=0; i< _numParticles; ++i)
	{
		m_particles.push_back(Particle(_pos,180,ngl::Vector(0,1,0)));
	}
	m_numParticles=_numParticles;

	ngl::Texture t(_texture);
	m_textureID=t.setTextureGL();
	// create the array for the particles we will update later
	m_particlePositions.resize(_numParticles*4);

}
/// @brief a method to update each of the particles contained in the system
void Emitter::update()
{
	for(int i=0; i<m_numParticles; ++i)
	{
		m_particles[i].update();
	}

}
/// @brief a method to draw all the particles contained in the system
void Emitter::draw()
{
 int index=-1;
 ngl::Vector pos;
 for(int i=0; i<m_numParticles; ++i)
 {
	 pos=m_particles[i].getPosition();
	 m_particlePositions[++index]=pos.m_x;
	 m_particlePositions[++index]=pos.m_y;
	 m_particlePositions[++index]=pos.m_z;
	 m_particlePositions[++index]=m_particles[i].getLife()/4.0;
 }

 glEnableClientState(GL_VERTEX_ARRAY);
 glVertexPointer(4, GL_FLOAT, 0, &m_particlePositions[0]);
 glDrawArraysInstancedARB(GL_POINTS, 0, m_numParticles,1);
 glDisableClientState(GL_VERTEX_ARRAY);

}
