#ifndef __GL_WINDOW_H__
#define __GL_WINDOW_H__


#include "ngl/Camera.h"
#include "ngl/TransformStack.h"
#include "Emitter.h"
#include <QtOpenGL>
#include <QTime>

/// @file GLWindow.h
/// @brief a basic Qt GL window class for ngl demos
/// @author Jonathan Macey
/// @version 1.0
/// @date 10/10/10
/// Revision History :
/// Initial Version 10/10/10 (Binary day ;-0 )
/// @class GLWindow
/// @brief our main glwindow widget for NGL applications all drawing elements are
/// put in this file
class GLWindow : public QGLWidget
{
Q_OBJECT        // must include this if you use Qt signals/slots
public :
  /// @brief Constructor for GLWindow
  /// @param [in] _parent the parent window to create the GL context in
  GLWindow(
           QWidget *_parent
          );
	void keyPress(
								QKeyEvent *_event
							 );

private :

  /// @brief Our Camera
  ngl::Camera *m_cam;
  /// @brief transformation stack for the gl transformations etc
  ngl::TransformStack m_transformStack;
	/// @brief our emitter
	Emitter *m_emitter;
	/// @brief the particle update timer
	int m_particleTimer;
	/// @brief
	ngl::Vector *m_wind;


protected:

  /// @brief  The following methods must be implimented in the sub class
  /// this is called when the window is created

  void initializeGL();

  /// @brief this is called whenever the window is re-sized
  /// @param[in] _w the width of the resized window
  /// @param[in] _h the height of the resized window

  void resizeGL(
                const int _w,
                const int _h
               );
  /// @brief this is the main gl drawing routine which is called whenever the window needs to
  // be re-drawn
  void paintGL();

private :

  /// @brief called when the timer is triggered

  void timerEvent(
                    QTimerEvent *_event
                   );
};

#endif
